package com.sd.framework.util;

import cn.hutool.core.io.IoUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.handler.WriteHandler;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.ruoyi.common.exception.ServiceException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.ClassPathResource;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class TemplateExcelUtil {

    /**
     * 自定义模板excel写入浏览器
     *
     * @param templateName 模板名称
     * @param fileName     文件名称
     * @param response     response
     * @param data         模板字段替换数据
     */
    public static void writeBrowser(String templateName, String fileName, HttpServletResponse response, Object... data) {

        ExcelWriter excelWriter = null;
        InputStream is = null;
        ServletOutputStream os = null;
        try {
            String fileNameCn = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);

            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileNameCn + ".xlsx");

            ClassPathResource classPathResource = new ClassPathResource("template" + File.separator + templateName);
            is = classPathResource.getInputStream();
            os = response.getOutputStream();

            excelWriter = EasyExcel.write(os).withTemplate(is).build();
            WriteSheet writeSheet = EasyExcel.writerSheet().build();
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

            for (Object item : data) {
                excelWriter.fill(item, fillConfig, writeSheet);
            }

        } catch (IOException e) {
            log.error("excel导出失败", e);
            throw new ServiceException("excel导出失败");
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
            IoUtil.close(excelWriter);
            IoUtil.close(is);
            IoUtil.close(os);
        }

    }

    /**
     * 自定义模板excel写入浏览器
     *
     * @param templateName 模板名称
     * @param sheetNum     写入sheet的数量
     * @param fileName     文件名称
     * @param response     response
     * @param data         模板字段替换数据
     */
    public static void writeBrowserMultiSheet(String templateName, int sheetNum, String fileName, HttpServletResponse response, Object... data) {

        ExcelWriter excelWriter = null;
        InputStream is = null;
        ServletOutputStream os = null;
        try {
            String fileNameCn = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);

            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileNameCn + ".xlsx");

            ClassPathResource classPathResource = new ClassPathResource("template" + File.separator + templateName);
            is = classPathResource.getInputStream();
            os = response.getOutputStream();

            excelWriter = EasyExcel.write(os).withTemplate(is).build();
            List<WriteSheet> writeSheetList = new ArrayList<>();
            for (int i = 0; i < sheetNum; i++) {
                writeSheetList.add(EasyExcel.writerSheet(i).build());
            }
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

            for (int i = 0; i < writeSheetList.size(); i++) {
                excelWriter.fill(data[i], fillConfig, writeSheetList.get(i));
            }

        } catch (IOException e) {
            log.error("excel导出失败", e);
            throw new ServiceException("excel导出失败");
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
            IoUtil.close(excelWriter);
            IoUtil.close(is);
            IoUtil.close(os);
        }

    }

    /**
     * 自定义模板excel写入浏览器
     *
     * @param templateName 模板名称
     * @param fileName     文件名称
     * @param writeHandler 合并策略
     * @param response     response
     * @param data         模板字段替换数据
     */
    public static void writeBrowser(String templateName, String fileName, WriteHandler writeHandler, HttpServletResponse response, Object... data) {

        ExcelWriter excelWriter = null;
        InputStream is = null;
        ServletOutputStream os = null;
        try {
            String fileNameCn = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);

            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileNameCn + ".xlsx");

            ClassPathResource classPathResource = new ClassPathResource("template" + File.separator + templateName);
            is = classPathResource.getInputStream();
            os = response.getOutputStream();

            excelWriter = EasyExcel.write(os).withTemplate(is).build();
            WriteSheet writeSheet = EasyExcel.writerSheet().registerWriteHandler(writeHandler).build();
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

            for (Object item : data) {
                excelWriter.fill(item, fillConfig, writeSheet);
            }

        } catch (IOException e) {
            log.error("excel导出失败", e);
            throw new ServiceException("excel导出失败");
        } finally {
            IoUtil.close(os);
            IoUtil.close(is);
            if (excelWriter != null) {
                excelWriter.finish();
            }
            IoUtil.close(excelWriter);
        }

    }

}
